@@ -1,17 +1,17 @@
-# Copyright 2013-2022 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2013-2024 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
 #    Reahl is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU Affero General Public License as
+#    it under the terms of the GNU Lesser General Public License as
 #    published by the Free Software Foundation; version 3 of the License.
 #
 #    This program is distributed in the hope that it will be useful,
 #    but WITHOUT ANY WARRANTY; without even the implied warranty of
 #    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU Affero General Public License for more details.
+#    GNU Lesser General Public License for more details.
 #
-#    You should have received a copy of the GNU Affero General Public License
+#    You should have received a copy of the GNU Lesser General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
@@ -43,9 +43,17 @@ from reahl.component.exceptions import ProgrammerError
 from reahl.component.context import ExecutionContext, NoContextFound
 from reahl.component.config import StoredConfiguration
 from reahl.component.shelltools import Executable
-from reahl.component.decorators import deprecated
 from reahl.web.fw import ReahlWSGIApplication
 
+import sys
+if sys.version_info < (3, 9):
+    try:
+        import importlib_resources
+    except:
+        raise Exception('You are on an older version of python. Please install importlib-resources')
+else:
+    import importlib.resources as importlib_resources
+
 
 class WrappedApp:
     """A class in which to wrap a WSGI app, allowing catching of exceptions in the wrapped app.
@@ -54,6 +62,7 @@ class WrappedApp:
         self.wrapped = wrapped
         self.exception = None
         self.traceback = None
+        self.ignore_exception = None
 
     def __call__(self, environ, start_response):
         app = self.wrapped
@@ -93,6 +102,7 @@ class WrappedApp:
     def clear_exception(self):
         self.exception = None
 
+        
 
 class NoopApp:
     def __init__(self, config=None):
@@ -230,7 +240,10 @@ class SSLCapableWSGIServer(ReahlWSGIServer):
         self.base_environ['HTTPS']='on'
 
     def server_bind(self):
-        self.socket = ssl.wrap_socket(self.socket, server_side=True, certfile=self.certfile)
+        context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
+        context.load_cert_chain(certfile=self.certfile)
+        self.socket = context.wrap_socket(self.socket, server_side=True)
+
         super().server_bind()
 
 
@@ -342,7 +355,7 @@ class ServerSupervisor(PatternMatchingEventHandler):
         self.request_stop_running = Event()
 
     #override for PatternMatchingEventHandler
-    def on_any_event(self, event):
+    def on_modified(self, event):
         self.files_changed.set()
 
     def start_observing_directories(self):
@@ -415,11 +428,6 @@ class ReahlWebServer:
 
         return cls(config)
 
-    @classmethod
-    @deprecated('renamed to from_config_directory','5.1')
-    def fromConfigDirectory(cls, directory, port):
-        return cls.from_config_directory(directory)
-
     def set_app(self, new_wsgi_app):
         """Changes the currently served application to `new_wsgi_app`."""
         self.reahl_wsgi_app.set_new_wrapped(new_wsgi_app)
@@ -436,17 +444,24 @@ class ReahlWebServer:
 
         self.port = int(config.web.default_http_port)
         self.encrypted_port = int(config.web.encrypted_http_port)
-        certfile = pkg_resources.resource_filename(__name__, 'reahl_development_cert.pem')
+        
+        self.certfile_manager = contextlib.ExitStack()
+        ref = importlib_resources.files(__package__) / 'reahl_development_cert.pem'
+        certfile_path = self.certfile_manager.enter_context(importlib_resources.as_file(ref))
+
         self.reahl_wsgi_app = WrappedApp(ReahlWSGIApplication(config, start_on_first_request=True))
         try:
             self.httpd = ReahlWSGIServer.make_server('', self.port, self.reahl_wsgi_app)
-            self.httpsd = SSLCapableWSGIServer.make_server('', self.encrypted_port, certfile, self.reahl_wsgi_app)
+            self.httpsd = SSLCapableWSGIServer.make_server('', self.encrypted_port, str(certfile_path), self.reahl_wsgi_app)
         except socket.error as ex:
             message = ('Caught socket.error: %s\nThis means that another process is using one of these ports: %s, %s. ' % (ex, self.port, self.encrypted_port)) \
-                     +'\nIf this happens while running tests, it probably means that a browser client did not close its side of a connection to a previous server you had running - and that the server socket now sits in TIME_WAIT state. Is there perhaps a browser hanging around from a previous run? I have no idea how to fix this automatically... see http://hea-www.harvard.edu/~fine/Tech/addrinuse.html'
+                    +'\nIf this happens while running tests, it probably means that a browser client did not close its side of a connection to a previous server you had running - and that the server socket now sits in TIME_WAIT state. Is there perhaps a browser hanging around from a previous run? I have no idea how to fix this automatically... see http://hea-www.harvard.edu/~fine/Tech/addrinuse.html'
 
             raise AssertionError(message)
 
+    def __del__(self):
+        self.certfile_manager.close()
+
     def main_loop(self, context=None):
 
         context = context or contextlib.nullcontext()
@@ -568,5 +583,3 @@ class ReahlWebServer:
             finally:
                 self.reinstall_handlers()
 
-
-
